const path = require("path");

const IS_PROD = process.env.NODE_ENV === "production";
const isDev = process.env.NODE_ENV === "development";

// CDN 链接配置
const CDN = {
    externals: {
        "element-plus": "ElementPlus",
        vue: "Vue",
        "vue-router": "VueRouter",
        vuex: "Vuex",
        axios: "axios",
    },
    build: {
        css: ["https://unpkg.com/element-plus/lib/theme-chalk/index.css"],
        js: [
            "https://cdn.bootcdn.net/ajax/libs/vue/3.0.11/vue.global.js",
            "https://cdn.bootcdn.net/ajax/libs/vue-router/4.0.6/vue-router.global.js",
            "https://cdn.bootcdn.net/ajax/libs/vuex/4.0.0/vuex.global.js",
            "https://cdn.bootcdn.net/ajax/libs/axios/0.21.1/axios.js",
            "https://cdn.bootcdn.net/ajax/libs/element-plus/1.0.2-beta.44/index.full.js",
            "https://cdn.bootcdn.net/ajax/libs/element-plus/1.0.2-beta.44/umd/locale/zh-cn.js",
            "https://unpkg.com/dayjs/locale/zh-cn.js",
        ],
    },
};

module.exports = {
    outputDir: isDev ? "./dist" : path.resolve("./public/dist/"),
    publicPath: isDev ? "/" : "/dist",
    lintOnSave: isDev,
    productionSourceMap: false, // 去掉生成环境的 sourceMap
    output: {
        filename: this.mode  === 'js/[name].[hash:10].js',
    },
    devServer: {
        proxy: "http://127.0.0.1:8055", //http://localhost:8055
        hot: true,
        disableHostCheck: true,
    },
    configureWebpack: (config) => {
        if (IS_PROD) {
            config.externals = CDN.externals;
        }

        //打包文件大小配置
        config["performance"] = {
            maxEntrypointSize: 10000000,
            maxAssetSize: 30000000,
        };
    },
    chainWebpack: (config) => {
        // 默认不开启 prefetch
        config.plugins.delete("prefetch");

        // 默认不开启 preload
        config.plugins.delete("preload");

        // 别名 alias
        config.resolve.alias.set("@", path.resolve("src"));

        // 别名 alias
        config.resolve.alias.set("%", path.resolve("server"));

        // 修改页面 title
        config.plugin("html").tap((args) => {
            args[0].title = "运营管理系统";
            if (IS_PROD) {
                args[0].cdn = CDN.build;
            }

            return args;
        });

        // 分包
        config.when(IS_PROD, (config) => {
            config.optimization.splitChunks({
                chunks: "all",
                cacheGroups: {
                    libs: {
                        name: "chunk-libs",
                        test: /[\\/]node_modules[\\/]/,
                        priority: 10,
                        chunks: "initial", // 只打包初始时依赖的第三方
                    },
                    elementPlus: {
                        name: "chunk-elementPlus", // 单独将 elementPlus 拆包
                        priority: 20, // 数字大权重到，满足多个 cacheGroups 的条件时候分到权重高的
                        test: /[\\/]node_modules[\\/]_?element-plus(.*)/,
                    },
                    commons: {
                        name: "chunk-commons",
                        test: path.resolve("src/components"),
                        minChunks: 3, // 被至少用三次以上打包分离
                        priority: 5, // 优先级
                        reuseExistingChunk: true, // 表示是否使用已有的 chunk，如果为 true 则表示如果当前的 chunk 包含的模块已经被抽取出去了，那么将不会重新生成新的。
                    },
                },
            });
        });

        config.optimization.runtimeChunk("single");
    },
};
